from typing import List

from _qwak_proto.qwak.build.v1.build_api_pb2 import (
    GetBuildRequest,
    GetBuildResponse,
    ListBuildsRequest,
    ListBuildsResponse,
    LogPhaseStatusRequest,
    LogPhaseStatusResponse,
    RegisterBuildRequest,
    RegisterBuildResponse,
    RegisterExperimentTrackingRequest,
    RegisterExperimentTrackingResponse,
    RegisterModelSchemaRequest,
    RegisterModelSchemaResponse,
    RegisterTagsRequest,
    RegisterTagsResponse,
    SaveFrameworkModelsRequest,
    SaveFrameworkModelsResponse,
    UpdateBuildStatusRequest,
    UpdateBuildStatusResponse,
)
from _qwak_proto.qwak.build.v1.build_api_pb2_grpc import BuildAPIServicer
from _qwak_proto.qwak.build.v1.build_pb2 import Audit, Build, BuildStatus
from google.protobuf.timestamp_pb2 import Timestamp
from qwak.clients.build_orchestrator.client import (
    FrameworkModelDataClass,
    HuggingModelDataClass,
)
from qwak_services_mock.mocks.utils.exception_handlers import raise_internal_grpc_error


class BuildOrchestratorBuildApiMock(BuildAPIServicer):
    def __init__(self):
        super(BuildOrchestratorBuildApiMock, self).__init__()
        self._builds: dict[str:Build] = {}
        self._framework_models: dict[str : List[FrameworkModelDataClass]] = {}

    def given_build(self, build: Build):
        self._builds[build.buildId] = build

    def SaveFrameworkModels(self, request: SaveFrameworkModelsRequest, context):
        self._framework_models[request.spec.build_id] = request.spec.framework_models
        return SaveFrameworkModelsResponse()

    def RegisterBuild(
        self, request: RegisterBuildRequest, context
    ) -> RegisterBuildResponse:
        timestamp = Timestamp()
        timestamp.GetCurrentTime()
        now = timestamp

        audit = Audit(
            created_by="", created_at=now, last_modified_by="", last_modified_at=now
        )
        self._builds[request.buildId] = Build(
            buildId=request.buildId,
            commitId=request.commitId,
            branchId="",
            buildConfig=request.buildConfig,
            build_status=BuildStatus.IN_PROGRESS,
            tags=request.tags,
            steps=request.steps,
            audit=audit,
        )

        return RegisterBuildResponse()

    def UpdateBuildStatus(
        self, request: UpdateBuildStatusRequest, context
    ) -> UpdateBuildStatusResponse:
        self._builds[request.buildId].build_status = request.build_status
        return UpdateBuildStatusResponse()

    def RegisterTags(
        self, request: RegisterTagsRequest, context
    ) -> RegisterTagsResponse:
        self._builds[request.build_id].tags.extend(request.tags)
        return RegisterTagsResponse()

    def RegisterExperimentTracking(
        self, request: RegisterExperimentTrackingRequest, context
    ) -> RegisterExperimentTrackingResponse:
        self._builds[request.build_id].params.update(request.params)
        self._builds[request.build_id].metrics.update(request.metrics)
        return RegisterExperimentTrackingResponse()

    def RegisterModelSchema(
        self, request: RegisterModelSchemaRequest, context
    ) -> RegisterModelSchemaResponse:
        self._builds[request.build_id].model_schema.CopyFrom(request.model_schema)
        return RegisterModelSchemaResponse()

    def GetBuild(self, request: GetBuildRequest, context) -> GetBuildResponse:
        return GetBuildResponse(build=self._builds[request.build_id])

    def ListBuilds(self, request: ListBuildsRequest, context) -> ListBuildsResponse:
        valid_builds = [
            self._builds[buildId]
            for buildId in self._builds
            if self._builds[buildId].model_uuid == request.model_uuid
        ]
        builds_to_return = []

        if len(request.filter.tags) > 0:
            for build in valid_builds:
                if request.filter.require_all_tags:
                    should_be_included = all(
                        tag in build.tags for tag in request.filter.tags
                    )
                else:
                    should_be_included = any(
                        tag in build.tags for tag in request.filter.tags
                    )
                if not request.filter.include_extra_tags:
                    should_be_included = should_be_included and not any(
                        tag not in request.filter.tags for tag in build.tags
                    )
                if should_be_included:
                    builds_to_return.append(build)
        else:
            builds_to_return = valid_builds

        return ListBuildsResponse(build=builds_to_return)

    def LogPhaseStatus(
        self, request: LogPhaseStatusRequest, context
    ) -> LogPhaseStatusResponse:
        try:
            if request.build_id not in self._builds:
                raise Exception(f"Build {request.build_id} doesn't exist")

            return LogPhaseStatusResponse()
        except Exception as e:
            raise_internal_grpc_error(context, e)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/features/feature_set_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.features import feature_set_state_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__state__pb2
from _qwak_proto.qwak.feature_store.features import feature_set_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__pb2
from _qwak_proto.qwak.feature_store.features import feature_set_types_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5qwak/feature_store/features/feature_set_service.proto\x12\x1bqwak.feature.store.features\x1a\x33qwak/feature_store/features/feature_set_state.proto\x1a-qwak/feature_store/features/feature_set.proto\x1a\x33qwak/feature_store/features/feature_set_types.proto\"b\n\x19RegisterFeatureSetRequest\x12\x45\n\x10\x66\x65\x61ture_set_spec\x18\x01 \x01(\x0b\x32+.qwak.feature.store.features.FeatureSetSpec\"Z\n\x1aRegisterFeatureSetResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"x\n\x17UpdateFeatureSetRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12\x45\n\x10\x66\x65\x61ture_set_spec\x18\x02 \x01(\x0b\x32+.qwak.feature.store.features.FeatureSetSpec\"X\n\x18UpdateFeatureSetResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"2\n\x18GetFeatureSetByIdRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\"Y\n\x19GetFeatureSetByIdResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"6\n\x1aGetFeatureSetByNameRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"[\n\x1bGetFeatureSetByNameResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\">\n#ListFeaturesetVersionsByNameRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"h\n$ListFeaturesetVersionsByNameResponse\x12@\n\x0b\x66\x65\x61turesets\x18\x01 \x03(\x0b\x32+.qwak.feature.store.features.FeaturesetInfo\"#\n!GetEnvToFeatureSetsMappingRequest\"\x91\x02\n\"GetEnvToFeatureSetsMappingResponse\x12\x7f\n\x1a\x65nv_to_feature_set_mapping\x18\x01 \x03(\x0b\x32[.qwak.feature.store.features.GetEnvToFeatureSetsMappingResponse.EnvToFeatureSetMappingEntry\x1aj\n\x1b\x45nvToFeatureSetMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.qwak.feature.store.features.FeatureSetList:\x02\x38\x01\"1\n\x17\x44\x65leteFeatureSetRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\"\x1a\n\x18\x44\x65leteFeatureSetResponse\"\\\n\x1e\x44\x65leteFeaturesetVersionRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61tureset_version_number\x18\x02 \x01(\x05\"!\n\x1f\x44\x65leteFeaturesetVersionResponse\"\x18\n\x16ListFeatureSetsRequest\"\\\n\x17ListFeatureSetsResponse\x12\x41\n\x10\x66\x65\x61ture_families\x18\x01 \x03(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"!\n\x1fListFeaturesetsByAccountRequest\"`\n ListFeaturesetsByAccountResponse\x12<\n\x0b\x66\x65\x61turesets\x18\x01 \x03(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"5\n\x19RunBatchFeatureSetRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"2\n\x1aRunBatchFeatureSetResponse\x12\x14\n\x0c\x65xecution_id\x18\x01 \x01(\t\"p\n RunParquetBatchFeatureSetRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\x12\x18\n\x10parquet_location\x18\x02 \x01(\t\x12\x18\n\x10timestamp_column\x18\x03 \x01(\t\"#\n!RunParquetBatchFeatureSetResponse\"2\n\x16\x43reateUploadURLRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"x\n\x18\x43reateUploadURLRequestV1\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\x12\x43\n\x0etransformation\x18\x02 \x01(\x0b\x32+.qwak.feature.store.features.Transformation\"I\n.CreateRealTimeFeatureExtractorUploadURLRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"-\n\x17\x43reateUploadURLResponse\x12\x12\n\nupload_url\x18\x01 \x01(\t\"\x89\x02\n\x1cUpdateFeatureSetStateRequest\x12m\n\x13\x66\x65\x61ture_names_state\x18\x01 \x03(\x0b\x32P.qwak.feature.store.features.UpdateFeatureSetStateRequest.FeatureNamesStateEntry\x12\x16\n\x0e\x65nvironment_id\x18\x02 \x01(\t\x1a\x62\n\x16\x46\x65\x61tureNamesStateEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.qwak.feature.store.features.StateUpdate:\x02\x38\x01\"e\n\x0bStateUpdate\x12V\n\x13\x66\x65\x61ture_names_state\x18\x01 \x01(\x0e\x32\x39.qwak.feature.store.features.FeatureSetLastExecutionState\"\x1f\n\x1dUpdateFeatureSetStateResponse\"K\n\x1fGetFeatureSetStateByNameRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\x12\x0e\n\x06run_id\x18\x02 \x01(\x03\"x\n GetFeatureSetStateByNameResponse\x12T\n\x11\x66\x65\x61ture_set_state\x18\x01 \x01(\x0e\x32\x39.qwak.feature.store.features.FeatureSetLastExecutionState\"9\n\x1dGetFeatureSetStateByIdRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"^\n\x1eGetFeatureSetStateByIdResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"1\n\x16PauseFeaturesetRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"\x19\n\x17PauseFeaturesetResponse\"2\n\x17ResumeFeaturesetRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"\x1a\n\x18ResumeFeaturesetResponse\">\n#GetFeaturesetSchedulingStateRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"m\n$GetFeaturesetSchedulingStateResponse\x12\x45\n\x05state\x18\x01 \x01(\x0e\x32\x36.qwak.feature.store.features.FeaturesetSchedulingState\"_\n!SetActiveFeaturesetVersionRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61tureset_version_number\x18\x02 \x01(\x05\"$\n\"SetActiveFeaturesetVersionResponse2\x81\x19\n\x11\x46\x65\x61tureSetService\x12\x85\x01\n\x12RegisterFeatureSet\x12\x36.qwak.feature.store.features.RegisterFeatureSetRequest\x1a\x37.qwak.feature.store.features.RegisterFeatureSetResponse\x12\x7f\n\x10UpdateFeatureSet\x12\x34.qwak.feature.store.features.UpdateFeatureSetRequest\x1a\x35.qwak.feature.store.features.UpdateFeatureSetResponse\x12\x82\x01\n\x11GetFeatureSetById\x12\x35.qwak.feature.store.features.GetFeatureSetByIdRequest\x1a\x36.qwak.feature.store.features.GetFeatureSetByIdResponse\x12\x88\x01\n\x13GetFeatureSetByName\x12\x37.qwak.feature.store.features.GetFeatureSetByNameRequest\x1a\x38.qwak.feature.store.features.GetFeatureSetByNameResponse\x12\xa3\x01\n\x1cListFeaturesetVersionsByName\x12@.qwak.feature.store.features.ListFeaturesetVersionsByNameRequest\x1a\x41.qwak.feature.store.features.ListFeaturesetVersionsByNameResponse\x12\x7f\n\x10\x44\x65leteFeatureSet\x12\x34.qwak.feature.store.features.DeleteFeatureSetRequest\x1a\x35.qwak.feature.store.features.DeleteFeatureSetResponse\x12\x94\x01\n\x17\x44\x65leteFeaturesetVersion\x12;.qwak.feature.store.features.DeleteFeaturesetVersionRequest\x1a<.qwak.feature.store.features.DeleteFeaturesetVersionResponse\x12|\n\x0fListFeatureSets\x12\x33.qwak.feature.store.features.ListFeatureSetsRequest\x1a\x34.qwak.feature.store.features.ListFeatureSetsResponse\x12\x97\x01\n\x18ListFeaturesetsByAccount\x12<.qwak.feature.store.features.ListFeaturesetsByAccountRequest\x1a=.qwak.feature.store.features.ListFeaturesetsByAccountResponse\x12\x85\x01\n\x12RunBatchFeatureSet\x12\x36.qwak.feature.store.features.RunBatchFeatureSetRequest\x1a\x37.qwak.feature.store.features.RunBatchFeatureSetResponse\x12\x9a\x01\n\x19RunParquetBatchFeatureSet\x12=.qwak.feature.store.features.RunParquetBatchFeatureSetRequest\x1a>.qwak.feature.store.features.RunParquetBatchFeatureSetResponse\x12\x81\x01\n\x0f\x43reateUploadURL\x12\x33.qwak.feature.store.features.CreateUploadURLRequest\x1a\x34.qwak.feature.store.features.CreateUploadURLResponse\"\x03\x88\x02\x01\x12\x80\x01\n\x11\x43reateUploadURLV1\x12\x35.qwak.feature.store.features.CreateUploadURLRequestV1\x1a\x34.qwak.feature.store.features.CreateUploadURLResponse\x12\xac\x01\n\'CreateRealTimeFeatureExtractorUploadURL\x12K.qwak.feature.store.features.CreateRealTimeFeatureExtractorUploadURLRequest\x1a\x34.qwak.feature.store.features.CreateUploadURLResponse\x12\x8e\x01\n\x15UpdateFeatureSetState\x12\x39.qwak.feature.store.features.UpdateFeatureSetStateRequest\x1a:.qwak.feature.store.features.UpdateFeatureSetStateResponse\x12\x97\x01\n\x18GetFeatureSetStateByName\x12<.qwak.feature.store.features.GetFeatureSetStateByNameRequest\x1a=.qwak.feature.store.features.GetFeatureSetStateByNameResponse\x12\x9d\x01\n\x1aGetEnvToFeatureSetsMapping\x12>.qwak.feature.store.features.GetEnvToFeatureSetsMappingRequest\x1a?.qwak.feature.store.features.GetEnvToFeatureSetsMappingResponse\x12\x91\x01\n\x16GetFeatureSetStateById\x12:.qwak.feature.store.features.GetFeatureSetStateByIdRequest\x1a;.qwak.feature.store.features.GetFeatureSetStateByIdResponse\x12|\n\x0fPauseFeatureset\x12\x33.qwak.feature.store.features.PauseFeaturesetRequest\x1a\x34.qwak.feature.store.features.PauseFeaturesetResponse\x12\x7f\n\x10ResumeFeatureset\x12\x34.qwak.feature.store.features.ResumeFeaturesetRequest\x1a\x35.qwak.feature.store.features.ResumeFeaturesetResponse\x12\xa3\x01\n\x1cGetFeaturesetSchedulingState\x12@.qwak.feature.store.features.GetFeaturesetSchedulingStateRequest\x1a\x41.qwak.feature.store.features.GetFeaturesetSchedulingStateResponse\x12\x9d\x01\n\x1aSetActiveFeaturesetVersion\x12>.qwak.feature.store.features.SetActiveFeaturesetVersionRequest\x1a?.qwak.feature.store.features.SetActiveFeaturesetVersionResponseB]\n(com.qwak.ai.feature.store.management.apiP\x01Z/qwak/featurestore/features;featurestorefeaturesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.feature_store.features.feature_set_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n(com.qwak.ai.feature.store.management.apiP\001Z/qwak/featurestore/features;featurestorefeatures'
  _GETENVTOFEATURESETSMAPPINGRESPONSE_ENVTOFEATURESETMAPPINGENTRY._options = None
  _GETENVTOFEATURESETSMAPPINGRESPONSE_ENVTOFEATURESETMAPPINGENTRY._serialized_options = b'8\001'
  _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY._options = None
  _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY._serialized_options = b'8\001'
  _FEATURESETSERVICE.methods_by_name['CreateUploadURL']._options = None
  _FEATURESETSERVICE.methods_by_name['CreateUploadURL']._serialized_options = b'\210\002\001'
  _globals['_REGISTERFEATURESETREQUEST']._serialized_start=239
  _globals['_REGISTERFEATURESETREQUEST']._serialized_end=337
  _globals['_REGISTERFEATURESETRESPONSE']._serialized_start=339
  _globals['_REGISTERFEATURESETRESPONSE']._serialized_end=429
  _globals['_UPDATEFEATURESETREQUEST']._serialized_start=431
  _globals['_UPDATEFEATURESETREQUEST']._serialized_end=551
  _globals['_UPDATEFEATURESETRESPONSE']._serialized_start=553
  _globals['_UPDATEFEATURESETRESPONSE']._serialized_end=641
  _globals['_GETFEATURESETBYIDREQUEST']._serialized_start=643
  _globals['_GETFEATURESETBYIDREQUEST']._serialized_end=693
  _globals['_GETFEATURESETBYIDRESPONSE']._serialized_start=695
  _globals['_GETFEATURESETBYIDRESPONSE']._serialized_end=784
  _globals['_GETFEATURESETBYNAMEREQUEST']._serialized_start=786
  _globals['_GETFEATURESETBYNAMEREQUEST']._serialized_end=840
  _globals['_GETFEATURESETBYNAMERESPONSE']._serialized_start=842
  _globals['_GETFEATURESETBYNAMERESPONSE']._serialized_end=933
  _globals['_LISTFEATURESETVERSIONSBYNAMEREQUEST']._serialized_start=935
  _globals['_LISTFEATURESETVERSIONSBYNAMEREQUEST']._serialized_end=997
  _globals['_LISTFEATURESETVERSIONSBYNAMERESPONSE']._serialized_start=999
  _globals['_LISTFEATURESETVERSIONSBYNAMERESPONSE']._serialized_end=1103
  _globals['_GETENVTOFEATURESETSMAPPINGREQUEST']._serialized_start=1105
  _globals['_GETENVTOFEATURESETSMAPPINGREQUEST']._serialized_end=1140
  _globals['_GETENVTOFEATURESETSMAPPINGRESPONSE']._serialized_start=1143
  _globals['_GETENVTOFEATURESETSMAPPINGRESPONSE']._serialized_end=1416
  _globals['_GETENVTOFEATURESETSMAPPINGRESPONSE_ENVTOFEATURESETMAPPINGENTRY']._serialized_start=1310
  _globals['_GETENVTOFEATURESETSMAPPINGRESPONSE_ENVTOFEATURESETMAPPINGENTRY']._serialized_end=1416
  _globals['_DELETEFEATURESETREQUEST']._serialized_start=1418
  _globals['_DELETEFEATURESETREQUEST']._serialized_end=1467
  _globals['_DELETEFEATURESETRESPONSE']._serialized_start=1469
  _globals['_DELETEFEATURESETRESPONSE']._serialized_end=1495
  _globals['_DELETEFEATURESETVERSIONREQUEST']._serialized_start=1497
  _globals['_DELETEFEATURESETVERSIONREQUEST']._serialized_end=1589
  _globals['_DELETEFEATURESETVERSIONRESPONSE']._serialized_start=1591
  _globals['_DELETEFEATURESETVERSIONRESPONSE']._serialized_end=1624
  _globals['_LISTFEATURESETSREQUEST']._serialized_start=1626
  _globals['_LISTFEATURESETSREQUEST']._serialized_end=1650
  _globals['_LISTFEATURESETSRESPONSE']._serialized_start=1652
  _globals['_LISTFEATURESETSRESPONSE']._serialized_end=1744
  _globals['_LISTFEATURESETSBYACCOUNTREQUEST']._serialized_start=1746
  _globals['_LISTFEATURESETSBYACCOUNTREQUEST']._serialized_end=1779
  _globals['_LISTFEATURESETSBYACCOUNTRESPONSE']._serialized_start=1781
  _globals['_LISTFEATURESETSBYACCOUNTRESPONSE']._serialized_end=1877
  _globals['_RUNBATCHFEATURESETREQUEST']._serialized_start=1879
  _globals['_RUNBATCHFEATURESETREQUEST']._serialized_end=1932
  _globals['_RUNBATCHFEATURESETRESPONSE']._serialized_start=1934
  _globals['_RUNBATCHFEATURESETRESPONSE']._serialized_end=1984
  _globals['_RUNPARQUETBATCHFEATURESETREQUEST']._serialized_start=1986
  _globals['_RUNPARQUETBATCHFEATURESETREQUEST']._serialized_end=2098
  _globals['_RUNPARQUETBATCHFEATURESETRESPONSE']._serialized_start=2100
  _globals['_RUNPARQUETBATCHFEATURESETRESPONSE']._serialized_end=2135
  _globals['_CREATEUPLOADURLREQUEST']._serialized_start=2137
  _globals['_CREATEUPLOADURLREQUEST']._serialized_end=2187
  _globals['_CREATEUPLOADURLREQUESTV1']._serialized_start=2189
  _globals['_CREATEUPLOADURLREQUESTV1']._serialized_end=2309
  _globals['_CREATEREALTIMEFEATUREEXTRACTORUPLOADURLREQUEST']._serialized_start=2311
  _globals['_CREATEREALTIMEFEATUREEXTRACTORUPLOADURLREQUEST']._serialized_end=2384
  _globals['_CREATEUPLOADURLRESPONSE']._serialized_start=2386
  _globals['_CREATEUPLOADURLRESPONSE']._serialized_end=2431
  _globals['_UPDATEFEATURESETSTATEREQUEST']._serialized_start=2434
  _globals['_UPDATEFEATURESETSTATEREQUEST']._serialized_end=2699
  _globals['_UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY']._serialized_start=2601
  _globals['_UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY']._serialized_end=2699
  _globals['_STATEUPDATE']._serialized_start=2701
  _globals['_STATEUPDATE']._serialized_end=2802
  _globals['_UPDATEFEATURESETSTATERESPONSE']._serialized_start=2804
  _globals['_UPDATEFEATURESETSTATERESPONSE']._serialized_end=2835
  _globals['_GETFEATURESETSTATEBYNAMEREQUEST']._serialized_start=2837
  _globals['_GETFEATURESETSTATEBYNAMEREQUEST']._serialized_end=2912
  _globals['_GETFEATURESETSTATEBYNAMERESPONSE']._serialized_start=2914
  _globals['_GETFEATURESETSTATEBYNAMERESPONSE']._serialized_end=3034
  _globals['_GETFEATURESETSTATEBYIDREQUEST']._serialized_start=3036
  _globals['_GETFEATURESETSTATEBYIDREQUEST']._serialized_end=3093
  _globals['_GETFEATURESETSTATEBYIDRESPONSE']._serialized_start=3095
  _globals['_GETFEATURESETSTATEBYIDRESPONSE']._serialized_end=3189
  _globals['_PAUSEFEATURESETREQUEST']._serialized_start=3191
  _globals['_PAUSEFEATURESETREQUEST']._serialized_end=3240
  _globals['_PAUSEFEATURESETRESPONSE']._serialized_start=3242
  _globals['_PAUSEFEATURESETRESPONSE']._serialized_end=3267
  _globals['_RESUMEFEATURESETREQUEST']._serialized_start=3269
  _globals['_RESUMEFEATURESETREQUEST']._serialized_end=3319
  _globals['_RESUMEFEATURESETRESPONSE']._serialized_start=3321
  _globals['_RESUMEFEATURESETRESPONSE']._serialized_end=3347
  _globals['_GETFEATURESETSCHEDULINGSTATEREQUEST']._serialized_start=3349
  _globals['_GETFEATURESETSCHEDULINGSTATEREQUEST']._serialized_end=3411
  _globals['_GETFEATURESETSCHEDULINGSTATERESPONSE']._serialized_start=3413
  _globals['_GETFEATURESETSCHEDULINGSTATERESPONSE']._serialized_end=3522
  _globals['_SETACTIVEFEATURESETVERSIONREQUEST']._serialized_start=3524
  _globals['_SETACTIVEFEATURESETVERSIONREQUEST']._serialized_end=3619
  _globals['_SETACTIVEFEATURESETVERSIONRESPONSE']._serialized_start=3621
  _globals['_SETACTIVEFEATURESETVERSIONRESPONSE']._serialized_end=3657
  _globals['_FEATURESETSERVICE']._serialized_start=3660
  _globals['_FEATURESETSERVICE']._serialized_end=6861
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.administration.runtime_configuration.v0.network_config_pb2
import qwak.administration.runtime_configuration.v0.runtime_config_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class EnvironmentRuntimeConfigSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RUNTIME_CONFIGURATION_FIELD_NUMBER: builtins.int
    NETWORK_CONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def runtime_configuration(self) -> qwak.administration.runtime_configuration.v0.runtime_config_pb2.RuntimeConfiguration: ...
    @property
    def network_configuration(self) -> qwak.administration.runtime_configuration.v0.network_config_pb2.EnvironmentNetworkConfiguration: ...
    def __init__(
        self,
        *,
        runtime_configuration: qwak.administration.runtime_configuration.v0.runtime_config_pb2.RuntimeConfiguration | None = ...,
        network_configuration: qwak.administration.runtime_configuration.v0.network_config_pb2.EnvironmentNetworkConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["network_configuration", b"network_configuration", "optional_runtime_config", b"optional_runtime_config", "runtime_configuration", b"runtime_configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["network_configuration", b"network_configuration", "optional_runtime_config", b"optional_runtime_config", "runtime_configuration", b"runtime_configuration"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["optional_runtime_config", b"optional_runtime_config"]) -> typing_extensions.Literal["runtime_configuration"] | None: ...

global___EnvironmentRuntimeConfigSpec = EnvironmentRuntimeConfigSpec



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/models/models_query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eqwak/models/models_query.proto\x12\x16qwak.models.management\"\x99\x01\n\nSortModels\x12\x31\n\x07sort_by\x18\x01 \x01(\x0e\x32\x1e.qwak.models.management.SortByH\x00\x12\x37\n\nsort_order\x18\x02 \x01(\x0e\x32!.qwak.models.management.SortOrderH\x01\x42\r\n\x0bsort_by_optB\x10\n\x0esort_order_opt\"\xc8\x01\n\x06\x46ilter\x12\x1e\n\x14model_name_substring\x18\x01 \x01(\tH\x00\x12\x19\n\x11model_group_names\x18\x02 \x03(\t\x12V\n\x17\x64\x65ployment_model_status\x18\x03 \x01(\x0e\x32\x33.qwak.models.management.DeploymentModelStatusFilterH\x01\x42\x0c\n\nsearch_optB\x1d\n\x1b\x64\x65ployment_model_status_opt\"<\n\x12PaginationMetadata\x12\x11\n\tpage_size\x18\x01 \x01(\x05\x12\x13\n\x0bnext_cursor\x18\x02 \x01(\t*J\n\x06SortBy\x12\x13\n\x0fSORT_BY_INVALID\x10\x00\x12\x10\n\x0cSORT_BY_NAME\x10\x01\x12\x19\n\x15SORT_BY_CREATION_DATE\x10\x02*L\n\tSortOrder\x12\x16\n\x12SORT_ORDER_INVALID\x10\x00\x12\x12\n\x0eSORT_ORDER_ASC\x10\x01\x12\x13\n\x0fSORT_ORDER_DESC\x10\x02*\xa5\x01\n\x1b\x44\x65ploymentModelStatusFilter\x12*\n&DEPLOYMENT_MODEL_STATUS_FILTER_INVALID\x10\x00\x12+\n\'DEPLOYMENT_MODEL_STATUS_FILTER_DEPLOYED\x10\x01\x12-\n)DEPLOYMENT_MODEL_STATUS_FILTER_UNDEPLOYED\x10\x02\x42\x36\n!com.qwak.ai.management.models.apiB\x0fModelQueryProtoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.models.models_query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n!com.qwak.ai.management.models.apiB\017ModelQueryProtoP\001'
  _globals['_SORTBY']._serialized_start=479
  _globals['_SORTBY']._serialized_end=553
  _globals['_SORTORDER']._serialized_start=555
  _globals['_SORTORDER']._serialized_end=631
  _globals['_DEPLOYMENTMODELSTATUSFILTER']._serialized_start=634
  _globals['_DEPLOYMENTMODELSTATUSFILTER']._serialized_end=799
  _globals['_SORTMODELS']._serialized_start=59
  _globals['_SORTMODELS']._serialized_end=212
  _globals['_FILTER']._serialized_start=215
  _globals['_FILTER']._serialized_end=415
  _globals['_PAGINATIONMETADATA']._serialized_start=417
  _globals['_PAGINATIONMETADATA']._serialized_end=477
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    CatalogSource as ProtoCatalogSource,
    UnityCatalogSource as ProtoUnityCatalogSource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.batch._batch import BaseBatchSource

RESERVED_CATALOG_NAME = "qwak_catalog"


@dataclass
class UnityCatalogSource(BaseBatchSource):
    """Unity Catalog batch data source.

    Attributes:
        uri (str): Connection uri to Unity Catalog i.e https://<workspace-instance-name>.<domain-name>/api/2.1/unity-catalog

        schema (str): The Unity Catalog schema name where the data is located.

        catalog (str): The Unity Catalog catalog name.

        personal_access_token_secret_name (Optional[str]): The name of the secret
            containing the personal access token for authentication. e.g "my-pat-secret"

        table (Optional[str]): The table name within the specified catalog and schema.
            Either 'table' or 'query' must be provided, but not both. e.g "my_table"

        query (Optional[str]): A SQL query to execute against the Unity Catalog.
            Either 'query' or 'table' must be provided, but not both. e.g "select * from {catalog}.{schema}.{table}"

        repository (Optional[str]): The data source group.

    """

    uri: str
    schema: str
    catalog: str
    personal_access_token_secret_name: Optional[str] = None
    table: Optional[str] = None
    query: Optional[str] = None
    repository: Optional[str] = None

    def __post_init__(self):
        self._validate()

    def _validate(self):
        if not self.uri:
            raise QwakException("`uri` cannot be empty.")

        if not self.schema:
            raise QwakException("`schema` cannot be empty.")

        if not self.catalog:
            raise QwakException("`catalog` cannot be empty.")

        if not self.personal_access_token_secret_name:
            raise QwakException("`personal_access_token_secret_name` cannot be empty.")

        if self.catalog == RESERVED_CATALOG_NAME:
            raise QwakException(
                f"`catalog` can not be set to `{RESERVED_CATALOG_NAME}`.",
            )

        if bool(self.query) == bool(self.table):
            raise QwakException("Either `query` or `table` must be provided.")

    def _to_proto(self, artifact_url: Optional[str] = None) -> ProtoDataSourceSpec:
        return ProtoDataSourceSpec(
            data_source_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                catalogSource=ProtoCatalogSource(
                    url=self.uri,
                    query=self.query,
                    table=self.table,
                    unityCatalogSource=ProtoUnityCatalogSource(
                        catalog=self.catalog,
                        schema=self.schema,
                        personal_access_token_secret_name=self.personal_access_token_secret_name,
                    ),
                ),
            ),
        )

    @classmethod
    def _from_proto(cls, proto: ProtoBatchSource) -> "UnityCatalogSource":
        catalog_source = proto.catalogSource
        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            uri=catalog_source.url,
            schema=catalog_source.unityCatalogSource.schema,
            catalog=catalog_source.unityCatalogSource.catalog,
            personal_access_token_secret_name=catalog_source.unityCatalogSource.personal_access_token_secret_name,
            table=catalog_source.table,
            query=catalog_source.query,
        )



UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG = (
    'please run: pip install -U "qwak-sdk[feature-store]"'
)


def missing_pyspark_exception_message(name: str) -> str:
    return (
        f"Missing 'pyspark' dependency required for {name} transformation. "
        f"{UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG}"
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/features_operator/v1/features_operator_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.features_operator.v1 import features_operator_pb2 as qwak_dot_features__operator_dot_v1_dot_features__operator__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9qwak/features_operator/v1/features_operator_service.proto\x12\x19qwak.features_operator.v1\x1a\x31qwak/features_operator/v1/features_operator.proto\"]\n\x15\x42\x61seValidationRequest\x12\x18\n\x10proto_definition\x18\x01 \x01(\t\x12\x0f\n\x07imports\x18\x02 \x01(\t\x12\x19\n\x11number_of_samples\x18\x03 \x01(\x05\"^\n\x19ValidateDataSourceRequest\x12\x41\n\x07request\x18\x01 \x01(\x0b\x32\x30.qwak.features_operator.v1.BaseValidationRequest\"w\n\x19ValidateFeatureSetRequest\x12\x41\n\x07request\x18\x01 \x01(\x0b\x32\x30.qwak.features_operator.v1.BaseValidationRequest\x12\x17\n\x0fs3_udf_zip_path\x18\x02 \x01(\t\"\xc0\x01\n\x12ValidationResponse\x12P\n\x10success_response\x18\x01 \x01(\x0b\x32\x34.qwak.features_operator.v1.ValidationSuccessResponseH\x00\x12P\n\x10\x66\x61ilure_response\x18\x02 \x01(\x0b\x32\x34.qwak.features_operator.v1.ValidationFailureResponseH\x00\x42\x06\n\x04type2\x8f\x02\n\x17\x46\x65\x61turesOperatorService\x12y\n\x12ValidateDataSource\x12\x34.qwak.features_operator.v1.ValidateDataSourceRequest\x1a-.qwak.features_operator.v1.ValidationResponse\x12y\n\x12ValidateFeatureSet\x12\x34.qwak.features_operator.v1.ValidateFeatureSetRequest\x1a-.qwak.features_operator.v1.ValidationResponseB(\n$com.qwak.ai.features.operator.api.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.features_operator.v1.features_operator_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.qwak.ai.features.operator.api.v1P\001'
  _globals['_BASEVALIDATIONREQUEST']._serialized_start=139
  _globals['_BASEVALIDATIONREQUEST']._serialized_end=232
  _globals['_VALIDATEDATASOURCEREQUEST']._serialized_start=234
  _globals['_VALIDATEDATASOURCEREQUEST']._serialized_end=328
  _globals['_VALIDATEFEATURESETREQUEST']._serialized_start=330
  _globals['_VALIDATEFEATURESETREQUEST']._serialized_end=449
  _globals['_VALIDATIONRESPONSE']._serialized_start=452
  _globals['_VALIDATIONRESPONSE']._serialized_end=644
  _globals['_FEATURESOPERATORSERVICE']._serialized_start=647
  _globals['_FEATURESOPERATORSERVICE']._serialized_end=918
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.admiral.secret.v0 import secret_service_pb2 as qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2


class SecretServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SetEnvironmentSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SecretService/SetEnvironmentSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.SetEnvironmentSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.SetEnvironmentSecretResponse.FromString,
                )
        self.DeleteEnvironmentSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SecretService/DeleteEnvironmentSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.DeleteEnvironmentSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.DeleteEnvironmentSecretResponse.FromString,
                )
        self.ListAccountSecrets = channel.unary_unary(
                '/qwak.admiral.secret.v0.SecretService/ListAccountSecrets',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.ListAccountSecretsRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.ListAccountSecretsResponse.FromString,
                )


class SecretServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SetEnvironmentSecret(self, request, context):
        """Set a value for a secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteEnvironmentSecret(self, request, context):
        """Remove a secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListAccountSecrets(self, request, context):
        """List account secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SecretServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SetEnvironmentSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.SetEnvironmentSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.SetEnvironmentSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.SetEnvironmentSecretResponse.SerializeToString,
            ),
            'DeleteEnvironmentSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteEnvironmentSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.DeleteEnvironmentSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.DeleteEnvironmentSecretResponse.SerializeToString,
            ),
            'ListAccountSecrets': grpc.unary_unary_rpc_method_handler(
                    servicer.ListAccountSecrets,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.ListAccountSecretsRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.ListAccountSecretsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.admiral.secret.v0.SecretService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SecretService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SetEnvironmentSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SecretService/SetEnvironmentSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.SetEnvironmentSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.SetEnvironmentSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteEnvironmentSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SecretService/DeleteEnvironmentSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.DeleteEnvironmentSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.DeleteEnvironmentSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListAccountSecrets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SecretService/ListAccountSecrets',
            qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.ListAccountSecretsRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_secret__service__pb2.ListAccountSecretsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/v0/environments/environment_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.administration.v0.environments import environment_pb2 as qwak_dot_administration_dot_v0_dot_environments_dot_environment__pb2
from _qwak_proto.qwak.administration.v0.environments import configuration_pb2 as qwak_dot_administration_dot_v0_dot_environments_dot_configuration__pb2
from _qwak_proto.qwak.administration.v0.environments import personalization_pb2 as qwak_dot_administration_dot_v0_dot_environments_dot_personalization__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=qwak/administration/v0/environments/environment_service.proto\x12\x1fqwak.administration.environment\x1a\x35qwak/administration/v0/environments/environment.proto\x1a\x37qwak/administration/v0/environments/configuration.proto\x1a\x39qwak/administration/v0/environments/personalization.proto\"\x9f\x02\n\x18\x43reateEnvironmentRequest\x12\x18\n\x10\x65nvironment_name\x18\x01 \x01(\t\x12\x15\n\ttenant_id\x18\x02 \x01(\tB\x02\x18\x01\x12\x12\n\naccount_id\x18\x04 \x01(\t\x12\x12\n\ncluster_id\x18\x06 \x01(\t\x12T\n\rconfiguration\x18\x03 \x01(\x0b\x32=.qwak.administration.environment.QwakEnvironmentConfiguration\x12T\n\x0fpersonalization\x18\x05 \x01(\x0b\x32;.qwak.administration.environment.EnvironmentPersonalization\"b\n\x19\x43reateEnvironmentResponse\x12\x45\n\x0b\x65nvironment\x18\x01 \x01(\x0b\x32\x30.qwak.administration.environment.QwakEnvironment\"\x95\x01\n%UpdateEnvironmentConfigurationRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\x12T\n\rconfiguration\x18\x02 \x01(\x0b\x32=.qwak.administration.environment.QwakEnvironmentConfiguration\"(\n&UpdateEnvironmentConfigurationResponse\"\x97\x01\n\'UpdateEnvironmentPersonalizationRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\x12T\n\x0fpersonalization\x18\x02 \x01(\x0b\x32;.qwak.administration.environment.EnvironmentPersonalization\"*\n(UpdateEnvironmentPersonalizationResponse\"/\n\x15GetEnvironmentRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\"_\n\x16GetEnvironmentResponse\x12\x45\n\x0b\x65nvironment\x18\x01 \x01(\x0b\x32\x30.qwak.administration.environment.QwakEnvironment\"I\n/GetEnvironmentApplicationUserCredentialsRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\"\x8f\x01\n0GetEnvironmentApplicationUserCredentialsResponse\x12[\n\x0b\x63redentials\x18\x01 \x01(\x0b\x32\x46.qwak.administration.environment.EnvironmentApplicationUserCredentials\"\xa6\x01\n/SetEnvironmentApplicationUserCredentialsRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\x12[\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32\x46.qwak.administration.environment.EnvironmentApplicationUserCredentials\"2\n0SetEnvironmentApplicationUserCredentialsResponse\"\xc8\x01\n\x17ListEnvironmentsRequest\x12O\n\x06\x66ilter\x18\x01 \x01(\x0b\x32?.qwak.administration.environment.ListEnvironmentsRequest.Filter\x1a\\\n\x06\x46ilter\x12R\n\x12\x65nvironment_status\x18\x01 \x01(\x0e\x32\x36.qwak.administration.environment.QwakEnvironmentStatus\"b\n\x18ListEnvironmentsResponse\x12\x46\n\x0c\x65nvironments\x18\x01 \x03(\x0b\x32\x30.qwak.administration.environment.QwakEnvironment2\xc1\t\n\x12\x45nvironmentService\x12\x8a\x01\n\x11\x43reateEnvironment\x12\x39.qwak.administration.environment.CreateEnvironmentRequest\x1a:.qwak.administration.environment.CreateEnvironmentResponse\x12\x81\x01\n\x0eGetEnvironment\x12\x36.qwak.administration.environment.GetEnvironmentRequest\x1a\x37.qwak.administration.environment.GetEnvironmentResponse\x12\xb1\x01\n\x1eUpdateEnvironmentConfiguration\x12\x46.qwak.administration.environment.UpdateEnvironmentConfigurationRequest\x1aG.qwak.administration.environment.UpdateEnvironmentConfigurationResponse\x12\xb7\x01\n UpdateEnvironmentPersonalization\x12H.qwak.administration.environment.UpdateEnvironmentPersonalizationRequest\x1aI.qwak.administration.environment.UpdateEnvironmentPersonalizationResponse\x12\x87\x01\n\x10ListEnvironments\x12\x38.qwak.administration.environment.ListEnvironmentsRequest\x1a\x39.qwak.administration.environment.ListEnvironmentsResponse\x12\xcf\x01\n(GetEnvironmentApplicationUserCredentials\x12P.qwak.administration.environment.GetEnvironmentApplicationUserCredentialsRequest\x1aQ.qwak.administration.environment.GetEnvironmentApplicationUserCredentialsResponse\x12\xcf\x01\n(SetEnvironmentApplicationUserCredentials\x12P.qwak.administration.environment.SetEnvironmentApplicationUserCredentialsRequest\x1aQ.qwak.administration.environment.SetEnvironmentApplicationUserCredentialsResponseB\xb2\x01\n\x1e\x63om.qwak.ai.administration.apiP\x01Z\x8d\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/v0/environments;environment_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.administration.v0.environments.environment_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036com.qwak.ai.administration.apiP\001Z\215\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/v0/environments;environment_v0'
  _CREATEENVIRONMENTREQUEST.fields_by_name['tenant_id']._options = None
  _CREATEENVIRONMENTREQUEST.fields_by_name['tenant_id']._serialized_options = b'\030\001'
  _globals['_CREATEENVIRONMENTREQUEST']._serialized_start=270
  _globals['_CREATEENVIRONMENTREQUEST']._serialized_end=557
  _globals['_CREATEENVIRONMENTRESPONSE']._serialized_start=559
  _globals['_CREATEENVIRONMENTRESPONSE']._serialized_end=657
  _globals['_UPDATEENVIRONMENTCONFIGURATIONREQUEST']._serialized_start=660
  _globals['_UPDATEENVIRONMENTCONFIGURATIONREQUEST']._serialized_end=809
  _globals['_UPDATEENVIRONMENTCONFIGURATIONRESPONSE']._serialized_start=811
  _globals['_UPDATEENVIRONMENTCONFIGURATIONRESPONSE']._serialized_end=851
  _globals['_UPDATEENVIRONMENTPERSONALIZATIONREQUEST']._serialized_start=854
  _globals['_UPDATEENVIRONMENTPERSONALIZATIONREQUEST']._serialized_end=1005
  _globals['_UPDATEENVIRONMENTPERSONALIZATIONRESPONSE']._serialized_start=1007
  _globals['_UPDATEENVIRONMENTPERSONALIZATIONRESPONSE']._serialized_end=1049
  _globals['_GETENVIRONMENTREQUEST']._serialized_start=1051
  _globals['_GETENVIRONMENTREQUEST']._serialized_end=1098
  _globals['_GETENVIRONMENTRESPONSE']._serialized_start=1100
  _globals['_GETENVIRONMENTRESPONSE']._serialized_end=1195
  _globals['_GETENVIRONMENTAPPLICATIONUSERCREDENTIALSREQUEST']._serialized_start=1197
  _globals['_GETENVIRONMENTAPPLICATIONUSERCREDENTIALSREQUEST']._serialized_end=1270
  _globals['_GETENVIRONMENTAPPLICATIONUSERCREDENTIALSRESPONSE']._serialized_start=1273
  _globals['_GETENVIRONMENTAPPLICATIONUSERCREDENTIALSRESPONSE']._serialized_end=1416
  _globals['_SETENVIRONMENTAPPLICATIONUSERCREDENTIALSREQUEST']._serialized_start=1419
  _globals['_SETENVIRONMENTAPPLICATIONUSERCREDENTIALSREQUEST']._serialized_end=1585
  _globals['_SETENVIRONMENTAPPLICATIONUSERCREDENTIALSRESPONSE']._serialized_start=1587
  _globals['_SETENVIRONMENTAPPLICATIONUSERCREDENTIALSRESPONSE']._serialized_end=1637
  _globals['_LISTENVIRONMENTSREQUEST']._serialized_start=1640
  _globals['_LISTENVIRONMENTSREQUEST']._serialized_end=1840
  _globals['_LISTENVIRONMENTSREQUEST_FILTER']._serialized_start=1748
  _globals['_LISTENVIRONMENTSREQUEST_FILTER']._serialized_end=1840
  _globals['_LISTENVIRONMENTSRESPONSE']._serialized_start=1842
  _globals['_LISTENVIRONMENTSRESPONSE']._serialized_end=1940
  _globals['_ENVIRONMENTSERVICE']._serialized_start=1943
  _globals['_ENVIRONMENTSERVICE']._serialized_end=3160
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/self_service/user/v1/user.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from _qwak_proto.qwak.administration.v0.users import user_pb2 as qwak_dot_administration_dot_v0_dot_users_dot_user__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$qwak/self_service/user/v1/user.proto\x12\x19qwak.self_service.user.v1\x1a\x1egoogle/protobuf/wrappers.proto\x1a\'qwak/administration/v0/users/user.proto\"\xdc\x02\n\x0bUserProfile\x12(\n\x02id\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x32\n\x0c\x64isplay_name\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x43\n\x0cjob_function\x18\x03 \x01(\x0e\x32-.qwak.administration.user.QwakUserJobFunction\x12=\n\tjob_level\x18\x04 \x01(\x0e\x32*.qwak.administration.user.QwakUserJobLevel\x12?\n\x0c\x61\x63\x63ount_role\x18\x05 \x01(\x0e\x32).qwak.administration.user.QwakAccountRole\x12\x17\n\x0f\x65nvironment_ids\x18\x06 \x03(\t\x12\x11\n\ttime_zone\x18\x07 \x01(\tB\xaf\x01\n\x1f\x63om.qwak.ai.self_service.v1.apiP\x01Z\x89\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/user/v1;self_service_user_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.self_service.user.v1.user_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.self_service.v1.apiP\001Z\211\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/user/v1;self_service_user_v1'
  _globals['_USERPROFILE']._serialized_start=141
  _globals['_USERPROFILE']._serialized_end=489
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from _qwak_proto.qwak.features_operator.v3.features_operator_async_service_pb2 import (
    DataSourceValidationOptions as ProtoDataSourceValidationOptions,
    FeatureSetValidationOptions as ProtoFeatureSetValidationOptions,
    ValidationTimeRange as ProtoValidationTimeRange,
)
from google.protobuf.timestamp_pb2 import Timestamp as ProtoTimestamp
from qwak.utils.datetime_utils import datetime_to_pts


@dataclass
class ValidationTimeRange:
    lower_time_bound: Optional[datetime] = None
    upper_time_bound: Optional[datetime] = None

    def to_proto(self) -> ProtoValidationTimeRange:
        lower_time_bound_proto: Optional[ProtoTimestamp] = None
        upper_time_bound_proto: Optional[ProtoTimestamp] = None

        if self.lower_time_bound:
            lower_time_bound_proto = datetime_to_pts(self.lower_time_bound)
        if self.upper_time_bound:
            upper_time_bound_proto = datetime_to_pts(self.upper_time_bound)

        return ProtoValidationTimeRange(
            lower_time_bound=lower_time_bound_proto,
            upper_time_bound=upper_time_bound_proto,
        )


@dataclass
class DataSourceValidationOptions:
    time_range: Optional[ValidationTimeRange] = field(
        default_factory=lambda: ValidationTimeRange()
    )

    def __init__(
        self,
        lower_time_bound: Optional[datetime] = None,
        upper_time_bound: Optional[datetime] = None,
    ):
        self.time_range = ValidationTimeRange(
            lower_time_bound=lower_time_bound, upper_time_bound=upper_time_bound
        )

    def to_proto(self) -> ProtoDataSourceValidationOptions:
        return ProtoDataSourceValidationOptions(
            validation_time_range=self.time_range.to_proto()
        )


@dataclass
class FeatureSetValidationOptions:
    time_range: Optional[ValidationTimeRange] = field(
        default_factory=lambda: ValidationTimeRange()
    )
    data_source_limit: Optional[int] = None

    def __init__(
        self,
        lower_time_bound: Optional[datetime] = None,
        upper_time_bound: Optional[datetime] = None,
        data_source_limit: Optional[int] = None,
    ):
        self.time_range = ValidationTimeRange(
            lower_time_bound=lower_time_bound, upper_time_bound=upper_time_bound
        )
        self.data_source_limit = data_source_limit

    def to_proto(self) -> ProtoFeatureSetValidationOptions:
        return ProtoFeatureSetValidationOptions(
            data_source_limit=self.data_source_limit,
            validation_time_range=self.time_range.to_proto(),
        )



import grpc
from _qwak_proto.qwak.feature_store.jobs.v1.job_pb2 import JobRecord, JobState
from _qwak_proto.qwak.feature_store.jobs.v1.job_service_pb2 import (
    ApplyJobRequest,
    ApplyJobResponse,
    GetLatestJobRequest,
    GetLatestJobResponse,
    GetLatestSuccessfulJobRequest,
    GetLatestSuccessfulJobResponse,
)
from _qwak_proto.qwak.feature_store.jobs.v1.job_service_pb2_grpc import (
    JobServiceServicer,
)
from qwak_services_mock.mocks.utils.exception_handlers import raise_internal_grpc_error


def _sort_by_running_id(job: JobRecord):
    return job.run_id


class JobRegistryServiceApiMock(JobServiceServicer):
    def __init__(self):
        self._job_records: dict[str:[JobRecord]] = {}
        self._running_id = 0
        super(JobRegistryServiceApiMock, self).__init__()

    def given_job(self, feature_set_id, job_id, job_state):
        new_job = JobRecord(
            job_id=job_id, featureset_id=feature_set_id, job_state=job_state
        )
        self._running_id += 1
        new_job.run_id = self._running_id

        self._job_records[new_job.featureset_id] = self._job_records.get(
            new_job.featureset_id, list()
        ) + [new_job]

    def GetLatestJob(self, request: GetLatestJobRequest, context):
        """
        Get latest job
        """
        try:
            records: list[JobRecord] = self._job_records.get(request.featureset_id)
            if records:
                records.sort(key=_sort_by_running_id, reverse=True)
                return GetLatestJobResponse(job=records[0])
            else:
                context.set_code(grpc.StatusCode.NOT_FOUND)
                context.set_details("Received a non existing job record")
                return GetLatestSuccessfulJobResponse()
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetLatestSuccessfulJob(self, request: GetLatestSuccessfulJobRequest, context):
        """
        Get latest successful job
        """
        try:
            records: list[JobRecord] = self._job_records.get(request.featureset_id)
            if records:
                records.sort(key=_sort_by_running_id, reverse=True)
                for job in records:
                    if job.job_state == JobState.COMPLETED:
                        return GetLatestSuccessfulJobResponse(job=job)
                return GetLatestSuccessfulJobResponse()
            else:
                context.set_code(grpc.StatusCode.NOT_FOUND)
                context.set_details("Received a non existing feature job record")
                return GetLatestSuccessfulJobResponse()
        except Exception as e:
            raise_internal_grpc_error(context, e)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/secret_service/secret_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(qwak/secret_service/secret_service.proto\x12\x13qwak.secret.service\"/\n\x10SetSecretRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x13\n\x11SetSecretResponse\" \n\x10GetSecretRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"0\n\x11GetSecretResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"#\n\x13\x44\x65leteSecretRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x16\n\x14\x44\x65leteSecretResponse2\xac\x02\n\rSecretService\x12Z\n\tSetSecret\x12%.qwak.secret.service.SetSecretRequest\x1a&.qwak.secret.service.SetSecretResponse\x12Z\n\tGetSecret\x12%.qwak.secret.service.GetSecretRequest\x1a&.qwak.secret.service.GetSecretResponse\x12\x63\n\x0c\x44\x65leteSecret\x12(.qwak.secret.service.DeleteSecretRequest\x1a).qwak.secret.service.DeleteSecretResponseB\"\n\x1e\x63om.qwak.ai.secret.service.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.secret_service.secret_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036com.qwak.ai.secret.service.apiP\001'
  _globals['_SETSECRETREQUEST']._serialized_start=65
  _globals['_SETSECRETREQUEST']._serialized_end=112
  _globals['_SETSECRETRESPONSE']._serialized_start=114
  _globals['_SETSECRETRESPONSE']._serialized_end=133
  _globals['_GETSECRETREQUEST']._serialized_start=135
  _globals['_GETSECRETREQUEST']._serialized_end=167
  _globals['_GETSECRETRESPONSE']._serialized_start=169
  _globals['_GETSECRETRESPONSE']._serialized_end=217
  _globals['_DELETESECRETREQUEST']._serialized_start=219
  _globals['_DELETESECRETREQUEST']._serialized_end=254
  _globals['_DELETESECRETRESPONSE']._serialized_start=256
  _globals['_DELETESECRETRESPONSE']._serialized_end=278
  _globals['_SECRETSERVICE']._serialized_start=281
  _globals['_SECRETSERVICE']._serialized_end=581
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.feature_store.features.aggregation_pb2
import qwak.feature_store.features.execution_pb2
import qwak.feature_store.features.monitoring_pb2
import qwak.feature_store.features.real_time_feature_extractor_pb2
import qwak.feature_store.sinks.sink_pb2
import qwak.feature_store.sources.batch_pb2
import qwak.feature_store.sources.streaming_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _FeatureSetTypeView:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FeatureSetTypeViewEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FeatureSetTypeView.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FEATURE_SET_TYPE_VIEW_INVALID: _FeatureSetTypeView.ValueType  # 0
    FEATURE_SET_TYPE_VIEW_BATCH: _FeatureSetTypeView.ValueType  # 1
    FEATURE_SET_TYPE_VIEW_STREAMING: _FeatureSetTypeView.ValueType  # 2

class FeatureSetTypeView(_FeatureSetTypeView, metaclass=_FeatureSetTypeViewEnumTypeWrapper): ...

FEATURE_SET_TYPE_VIEW_INVALID: FeatureSetTypeView.ValueType  # 0
FEATURE_SET_TYPE_VIEW_BATCH: FeatureSetTypeView.ValueType  # 1
FEATURE_SET_TYPE_VIEW_STREAMING: FeatureSetTypeView.ValueType  # 2
global___FeatureSetTypeView = FeatureSetTypeView

class FeatureSetType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BATCH_FEATURE_SET_FIELD_NUMBER: builtins.int
    ON_THE_FLY_FEATURE_SET_FIELD_NUMBER: builtins.int
    STREAMING_FEATURE_SET_FIELD_NUMBER: builtins.int
    STREAMING_FEATURE_SET_V1_FIELD_NUMBER: builtins.int
    BATCH_FEATURE_SET_V1_FIELD_NUMBER: builtins.int
    STREAMING_AGGREGATION_FEATURE_SET_FIELD_NUMBER: builtins.int
    @property
    def batch_feature_set(self) -> global___BatchFeatureSet: ...
    @property
    def on_the_fly_feature_set(self) -> global___OnTheFlyFeatureSet: ...
    @property
    def streaming_feature_set(self) -> global___StreamingFeatureSet: ...
    @property
    def streaming_feature_set_v1(self) -> global___StreamingFeatureSetV1: ...
    @property
    def batch_feature_set_v1(self) -> global___BatchFeatureSetV1: ...
    @property
    def streaming_aggregation_feature_set(self) -> global___StreamingAggregationFeatureSet: ...
    def __init__(
        self,
        *,
        batch_feature_set: global___BatchFeatureSet | None = ...,
        on_the_fly_feature_set: global___OnTheFlyFeatureSet | None = ...,
        streaming_feature_set: global___StreamingFeatureSet | None = ...,
        streaming_feature_set_v1: global___StreamingFeatureSetV1 | None = ...,
        batch_feature_set_v1: global___BatchFeatureSetV1 | None = ...,
        streaming_aggregation_feature_set: global___StreamingAggregationFeatureSet | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_feature_set", b"batch_feature_set", "batch_feature_set_v1", b"batch_feature_set_v1", "on_the_fly_feature_set", b"on_the_fly_feature_set", "set_type", b"set_type", "streaming_aggregation_feature_set", b"streaming_aggregation_feature_set", "streaming_feature_set", b"streaming_feature_set", "streaming_feature_set_v1", b"streaming_feature_set_v1"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_feature_set", b"batch_feature_set", "batch_feature_set_v1", b"batch_feature_set_v1", "on_the_fly_feature_set", b"on_the_fly_feature_set", "set_type", b"set_type", "streaming_aggregation_feature_set", b"streaming_aggregation_feature_set", "streaming_feature_set", b"streaming_feature_set", "streaming_feature_set_v1", b"streaming_feature_set_v1"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["set_type", b"set_type"]) -> typing_extensions.Literal["batch_feature_set", "on_the_fly_feature_set", "streaming_feature_set", "streaming_feature_set_v1", "batch_feature_set_v1", "streaming_aggregation_feature_set"] | None: ...

global___FeatureSetType = FeatureSetType

class BatchFeatureSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCHEDULING_POLICY_FIELD_NUMBER: builtins.int
    DATA_SOURCES_FIELD_NUMBER: builtins.int
    BACKFILL_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    FEATURE_SET_BATCH_SOURCES_FIELD_NUMBER: builtins.int
    MONITORING_CONFIGURATIONS_FIELD_NUMBER: builtins.int
    scheduling_policy: builtins.str
    """Scheduling policy of the resulting ETL that will generate data for this feature"""
    @property
    def data_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.sources.batch_pb2.BatchSource]:
        """Data sources used by the feature set
        Will assume the DataSourceReadPolicy of these sources are of type ReadOnce
        """
    @property
    def backfill(self) -> global___Backfill:
        """Backfill strategy"""
    @property
    def function(self) -> global___Function:
        """Actual function code"""
    @property
    def feature_set_batch_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FeatureSetBatchSource]:
        """Data sources and their properties which are used by the feature set"""
    @property
    def monitoring_configurations(self) -> qwak.feature_store.features.monitoring_pb2.MonitoringConfigurations:
        """Monitoring configuration"""
    def __init__(
        self,
        *,
        scheduling_policy: builtins.str = ...,
        data_sources: collections.abc.Iterable[qwak.feature_store.sources.batch_pb2.BatchSource] | None = ...,
        backfill: global___Backfill | None = ...,
        function: global___Function | None = ...,
        feature_set_batch_sources: collections.abc.Iterable[global___FeatureSetBatchSource] | None = ...,
        monitoring_configurations: qwak.feature_store.features.monitoring_pb2.MonitoringConfigurations | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "function", b"function", "monitoring_configurations", b"monitoring_configurations"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "data_sources", b"data_sources", "feature_set_batch_sources", b"feature_set_batch_sources", "function", b"function", "monitoring_configurations", b"monitoring_configurations", "scheduling_policy", b"scheduling_policy"]) -> None: ...

global___BatchFeatureSet = BatchFeatureSet

class BatchFeatureSetV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCHEDULING_POLICY_FIELD_NUMBER: builtins.int
    BACKFILL_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    FEATURE_SET_BATCH_SOURCES_FIELD_NUMBER: builtins.int
    EXECUTION_SPEC_FIELD_NUMBER: builtins.int
    OFFLINE_SINK_FIELD_NUMBER: builtins.int
    ONLINE_SINK_FIELD_NUMBER: builtins.int
    MONITORING_CONFIGURATIONS_FIELD_NUMBER: builtins.int
    TIMESTAMP_COLUMN_NAME_FIELD_NUMBER: builtins.int
    TRANSFORMATION_FIELD_NUMBER: builtins.int
    QWAK_INTERNAL_PROTOCOL_VERSION_FIELD_NUMBER: builtins.int
    REAL_TIME_FEATURE_EXTRACTOR_FIELD_NUMBER: builtins.int
    DATA_RETENTION_OPTIONS_FIELD_NUMBER: builtins.int
    scheduling_policy: builtins.str
    """Scheduling policy of batch feature set. Can be a cron string format or @annotation: @daily, @monthly, @yearly"""
    @property
    def backfill(self) -> global___Backfill:
        """Backfill strategy"""
    @property
    def function(self) -> global___Function:
        """Actual function code"""
    @property
    def feature_set_batch_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FeatureSetBatchSource]:
        """Data sources and their properties which are used by the feature set"""
    @property
    def execution_spec(self) -> qwak.feature_store.features.execution_pb2.ExecutionSpec:
        """Execution Spec of the FeatureSet"""
    offline_sink: builtins.bool
    """Is Feature set has offline store"""
    online_sink: builtins.bool
    """Is Feature set has online store"""
    @property
    def monitoring_configurations(self) -> qwak.feature_store.features.monitoring_pb2.MonitoringConfigurations:
        """Monitoring configuration"""
    timestamp_column_name: builtins.str
    """Name of the timestamp column (must exist in the result), i.e: event time column"""
    @property
    def transformation(self) -> global___Transformation:
        """Transformation"""
    qwak_internal_protocol_version: builtins.int
    """Qwak featureset version"""
    @property
    def real_time_feature_extractor(self) -> qwak.feature_store.features.real_time_feature_extractor_pb2.RealTimeFeatureExtractor: ...
    @property
    def data_retention_options(self) -> global___DataRetentionOptions: ...
    def __init__(
        self,
        *,
        scheduling_policy: builtins.str = ...,
        backfill: global___Backfill | None = ...,
        function: global___Function | None = ...,
        feature_set_batch_sources: collections.abc.Iterable[global___FeatureSetBatchSource] | None = ...,
        execution_spec: qwak.feature_store.features.execution_pb2.ExecutionSpec | None = ...,
        offline_sink: builtins.bool = ...,
        online_sink: builtins.bool = ...,
        monitoring_configurations: qwak.feature_store.features.monitoring_pb2.MonitoringConfigurations | None = ...,
        timestamp_column_name: builtins.str = ...,
        transformation: global___Transformation | None = ...,
        qwak_internal_protocol_version: builtins.int = ...,
        real_time_feature_extractor: qwak.feature_store.features.real_time_feature_extractor_pb2.RealTimeFeatureExtractor | None = ...,
        data_retention_options: global___DataRetentionOptions | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "data_retention_options", b"data_retention_options", "execution_spec", b"execution_spec", "feature_extractor", b"feature_extractor", "function", b"function", "monitoring_configurations", b"monitoring_configurations", "real_time_feature_extractor", b"real_time_feature_extractor", "transformation", b"transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "data_retention_options", b"data_retention_options", "execution_spec", b"execution_spec", "feature_extractor", b"feature_extractor", "feature_set_batch_sources", b"feature_set_batch_sources", "function", b"function", "monitoring_configurations", b"monitoring_configurations", "offline_sink", b"offline_sink", "online_sink", b"online_sink", "qwak_internal_protocol_version", b"qwak_internal_protocol_version", "real_time_feature_extractor", b"real_time_feature_extractor", "scheduling_policy", b"scheduling_policy", "timestamp_column_name", b"timestamp_column_name", "transformation", b"transformation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["feature_extractor", b"feature_extractor"]) -> typing_extensions.Literal["real_time_feature_extractor"] | None: ...

global___BatchFeatureSetV1 = BatchFeatureSetV1

class DataRetentionOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ONLINE_TIME_TO_LIVE_SECONDS_FIELD_NUMBER: builtins.int
    online_time_to_live_seconds: builtins.int
    """Time to live per entity after event-time in the online store, in seconds."""
    def __init__(
        self,
        *,
        online_time_to_live_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["online_time_to_live_seconds", b"online_time_to_live_seconds"]) -> None: ...

global___DataRetentionOptions = DataRetentionOptions

class FeatureSetBatchSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_FIELD_NUMBER: builtins.int
    READ_POLICY_FIELD_NUMBER: builtins.int
    @property
    def data_source(self) -> qwak.feature_store.sources.batch_pb2.BatchSource:
        """Batch data source definition"""
    @property
    def read_policy(self) -> global___DataSourceReadPolicy:
        """Read policy of this batch source"""
    def __init__(
        self,
        *,
        data_source: qwak.feature_store.sources.batch_pb2.BatchSource | None = ...,
        read_policy: global___DataSourceReadPolicy | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data_source", b"data_source", "read_policy", b"read_policy"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source", b"data_source", "read_policy", b"read_policy"]) -> None: ...

global___FeatureSetBatchSource = FeatureSetBatchSource

class StreamingFeatureSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCES_FIELD_NUMBER: builtins.int
    @property
    def data_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.sources.streaming_pb2.StreamingSource]:
        """Data sources used by the feature"""
    def __init__(
        self,
        *,
        data_sources: collections.abc.Iterable[qwak.feature_store.sources.streaming_pb2.StreamingSource] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_sources", b"data_sources"]) -> None: ...

global___StreamingFeatureSet = StreamingFeatureSet

class StreamingFeatureSetV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCES_FIELD_NUMBER: builtins.int
    TRANSFORMATION_FIELD_NUMBER: builtins.int
    EXECUTION_SPEC_FIELD_NUMBER: builtins.int
    TIMESTAMP_COLUMN_NAME_FIELD_NUMBER: builtins.int
    ONLINE_TRIGGER_INTERVAL_FIELD_NUMBER: builtins.int
    OFFLINE_SCHEDULING_POLICY_FIELD_NUMBER: builtins.int
    QWAK_INTERNAL_PROTOCOL_VERSION_FIELD_NUMBER: builtins.int
    AUXILIARY_SINKS_FIELD_NUMBER: builtins.int
    @property
    def data_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.sources.streaming_pb2.StreamingSource]:
        """Data sources used by the FeatureSet"""
    @property
    def transformation(self) -> global___Transformation:
        """transformation flavour"""
    @property
    def execution_spec(self) -> qwak.feature_store.features.execution_pb2.StreamingExecutionSpec:
        """Execution Spec of the FeatureSet"""
    timestamp_column_name: builtins.str
    """Name of the timestamp column (must exist in the result),
    used for creating a total order over the events
    """
    online_trigger_interval: builtins.int
    """Trigger interval for the Online, in seconds"""
    offline_scheduling_policy: builtins.str
    """scheduling policy for the offline jobs"""
    qwak_internal_protocol_version: builtins.int
    """Qwak featureset version"""
    @property
    def auxiliary_sinks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.sinks.sink_pb2.StreamingSink]:
        """Auxiliary Sinks defined for this Featureset"""
    def __init__(
        self,
        *,
        data_sources: collections.abc.Iterable[qwak.feature_store.sources.streaming_pb2.StreamingSource] | None = ...,
        transformation: global___Transformation | None = ...,
        execution_spec: qwak.feature_store.features.execution_pb2.StreamingExecutionSpec | None = ...,
        timestamp_column_name: builtins.str = ...,
        online_trigger_interval: builtins.int = ...,
        offline_scheduling_policy: builtins.str = ...,
        qwak_internal_protocol_version: builtins.int = ...,
        auxiliary_sinks: collections.abc.Iterable[qwak.feature_store.sinks.sink_pb2.StreamingSink] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["execution_spec", b"execution_spec", "transformation", b"transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["auxiliary_sinks", b"auxiliary_sinks", "data_sources", b"data_sources", "execution_spec", b"execution_spec", "offline_scheduling_policy", b"offline_scheduling_policy", "online_trigger_interval", b"online_trigger_interval", "qwak_internal_protocol_version", b"qwak_internal_protocol_version", "timestamp_column_name", b"timestamp_column_name", "transformation", b"transformation"]) -> None: ...

global___StreamingFeatureSetV1 = StreamingFeatureSetV1

class StreamingAggregationFeatureSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCES_FIELD_NUMBER: builtins.int
    TRANSFORMATION_FIELD_NUMBER: builtins.int
    EXECUTION_SPEC_FIELD_NUMBER: builtins.int
    TIMESTAMP_COLUMN_NAME_FIELD_NUMBER: builtins.int
    ONLINE_TRIGGER_INTERVAL_FIELD_NUMBER: builtins.int
    COMPACTION_SCHEDULING_POLICY_FIELD_NUMBER: builtins.int
    AGGREGATION_SPEC_FIELD_NUMBER: builtins.int
    BACKFILL_SPEC_FIELD_NUMBER: builtins.int
    QWAK_INTERNAL_PROTOCOL_VERSION_FIELD_NUMBER: builtins.int
    BACKFILL_MAX_TIMESTAMP_FIELD_NUMBER: builtins.int
    @property
    def data_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.sources.streaming_pb2.StreamingSource]:
        """Data sources used by the FeatureSet"""
    @property
    def transformation(self) -> global___Transformation:
        """transformation flavour"""
    @property
    def execution_spec(self) -> qwak.feature_store.features.execution_pb2.StreamingExecutionSpec:
        """Execution Spec of the FeatureSet"""
    timestamp_column_name: builtins.str
    """Name of the timestamp column (must exist in the result),
    used for creating a total order over the events
    """
    online_trigger_interval: builtins.int
    """Trigger interval for the Online, in seconds"""
    compaction_scheduling_policy: builtins.str
    """scheduling policy for the compaction job, cron expression"""
    @property
    def aggregation_spec(self) -> global___AggregationSpec:
        """Aggregation spec for the feature set"""
    @property
    def backfill_spec(self) -> global___BackfillSpec:
        """Backfill spec"""
    qwak_internal_protocol_version: builtins.int
    """Qwak featureset version"""
    @property
    def backfill_max_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Event-time cutoff point between backfill jobs and compaction/row-level jobs"""
    def __init__(
        self,
        *,
        data_sources: collections.abc.Iterable[qwak.feature_store.sources.streaming_pb2.StreamingSource] | None = ...,
        transformation: global___Transformation | None = ...,
        execution_spec: qwak.feature_store.features.execution_pb2.StreamingExecutionSpec | None = ...,
        timestamp_column_name: builtins.str = ...,
        online_trigger_interval: builtins.int = ...,
        compaction_scheduling_policy: builtins.str = ...,
        aggregation_spec: global___AggregationSpec | None = ...,
        backfill_spec: global___BackfillSpec | None = ...,
        qwak_internal_protocol_version: builtins.int = ...,
        backfill_max_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aggregation_spec", b"aggregation_spec", "backfill_max_timestamp", b"backfill_max_timestamp", "backfill_spec", b"backfill_spec", "execution_spec", b"execution_spec", "transformation", b"transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregation_spec", b"aggregation_spec", "backfill_max_timestamp", b"backfill_max_timestamp", "backfill_spec", b"backfill_spec", "compaction_scheduling_policy", b"compaction_scheduling_policy", "data_sources", b"data_sources", "execution_spec", b"execution_spec", "online_trigger_interval", b"online_trigger_interval", "qwak_internal_protocol_version", b"qwak_internal_protocol_version", "timestamp_column_name", b"timestamp_column_name", "transformation", b"transformation"]) -> None: ...

global___StreamingAggregationFeatureSet = StreamingAggregationFeatureSet

class BackfillBatchDataSourceSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_FIELD_NUMBER: builtins.int
    START_TIMESTAMP_FIELD_NUMBER: builtins.int
    END_TIMESTAMP_FIELD_NUMBER: builtins.int
    @property
    def data_source(self) -> qwak.feature_store.sources.batch_pb2.BatchSource:
        """Batch data source"""
    @property
    def start_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Optional start timestamp"""
    @property
    def end_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Optional end timestamp"""
    def __init__(
        self,
        *,
        data_source: qwak.feature_store.sources.batch_pb2.BatchSource | None = ...,
        start_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data_source", b"data_source", "end_timestamp", b"end_timestamp", "start_timestamp", b"start_timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source", b"data_source", "end_timestamp", b"end_timestamp", "start_timestamp", b"start_timestamp"]) -> None: ...

global___BackfillBatchDataSourceSpec = BackfillBatchDataSourceSpec

class BackfillDataSourceSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BATCH_DATA_SOURCE_SPEC_FIELD_NUMBER: builtins.int
    @property
    def batch_data_source_spec(self) -> global___BackfillBatchDataSourceSpec: ...
    def __init__(
        self,
        *,
        batch_data_source_spec: global___BackfillBatchDataSourceSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_data_source_spec", b"batch_data_source_spec", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_data_source_spec", b"batch_data_source_spec", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["batch_data_source_spec"] | None: ...

global___BackfillDataSourceSpec = BackfillDataSourceSpec

class BackfillSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTION_SPEC_FIELD_NUMBER: builtins.int
    START_TIMESTAMP_FIELD_NUMBER: builtins.int
    END_TIMESTAMP_FIELD_NUMBER: builtins.int
    TRANSFORMATION_FIELD_NUMBER: builtins.int
    DATA_SOURCE_SPECS_FIELD_NUMBER: builtins.int
    @property
    def execution_spec(self) -> qwak.feature_store.features.execution_pb2.BackfillExecutionSpec:
        """Backfill resource spec"""
    @property
    def start_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Backfill start timestamp"""
    @property
    def end_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Backfill end timestamp"""
    @property
    def transformation(self) -> global___Transformation:
        """Transformation"""
    @property
    def data_source_specs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BackfillDataSourceSpec]: ...
    def __init__(
        self,
        *,
        execution_spec: qwak.feature_store.features.execution_pb2.BackfillExecutionSpec | None = ...,
        start_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        transformation: global___Transformation | None = ...,
        data_source_specs: collections.abc.Iterable[global___BackfillDataSourceSpec] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["end_timestamp", b"end_timestamp", "execution_spec", b"execution_spec", "start_timestamp", b"start_timestamp", "transformation", b"transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source_specs", b"data_source_specs", "end_timestamp", b"end_timestamp", "execution_spec", b"execution_spec", "start_timestamp", b"start_timestamp", "transformation", b"transformation"]) -> None: ...

global___BackfillSpec = BackfillSpec

class AggregationSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AGGREGATIONS_FIELD_NUMBER: builtins.int
    SLIDE_SECONDS_FIELD_NUMBER: builtins.int
    ALLOWED_LATE_ARRIVAL_SECONDS_FIELD_NUMBER: builtins.int
    @property
    def aggregations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.features.aggregation_pb2.AggregationField]:
        """List of aggregations"""
    slide_seconds: builtins.int
    """Aggregation slide in seconds"""
    allowed_late_arrival_seconds: builtins.int
    """Watermark"""
    def __init__(
        self,
        *,
        aggregations: collections.abc.Iterable[qwak.feature_store.features.aggregation_pb2.AggregationField] | None = ...,
        slide_seconds: builtins.int = ...,
        allowed_late_arrival_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregations", b"aggregations", "allowed_late_arrival_seconds", b"allowed_late_arrival_seconds", "slide_seconds", b"slide_seconds"]) -> None: ...

global___AggregationSpec = AggregationSpec

class Transformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UDF_TRANSFORMATION_FIELD_NUMBER: builtins.int
    SQL_TRANSFORMATION_FIELD_NUMBER: builtins.int
    KOALAS_TRANSFORMATION_FIELD_NUMBER: builtins.int
    PYSPARK_TRANSFORMATION_FIELD_NUMBER: builtins.int
    PANDAS_ON_SPARK_TRANSFORMATION_FIELD_NUMBER: builtins.int
    ARTIFACT_PATH_FIELD_NUMBER: builtins.int
    @property
    def udf_transformation(self) -> global___UdfTransformation:
        """UDF Transformation - a pandas_udf that gets the entire row"""
    @property
    def sql_transformation(self) -> global___SqlTransformation:
        """User provided sql"""
    @property
    def koalas_transformation(self) -> global___KoalasTransformation:
        """Koalas provided transformation"""
    @property
    def pyspark_transformation(self) -> global___PySparkTransformation:
        """PySpark provided transformation"""
    @property
    def pandas_on_spark_transformation(self) -> global___PandasOnSparkTransformation:
        """Pandas provided transformation"""
    artifact_path: builtins.str
    def __init__(
        self,
        *,
        udf_transformation: global___UdfTransformation | None = ...,
        sql_transformation: global___SqlTransformation | None = ...,
        koalas_transformation: global___KoalasTransformation | None = ...,
        pyspark_transformation: global___PySparkTransformation | None = ...,
        pandas_on_spark_transformation: global___PandasOnSparkTransformation | None = ...,
        artifact_path: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["koalas_transformation", b"koalas_transformation", "pandas_on_spark_transformation", b"pandas_on_spark_transformation", "pyspark_transformation", b"pyspark_transformation", "sql_transformation", b"sql_transformation", "type", b"type", "udf_transformation", b"udf_transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["artifact_path", b"artifact_path", "koalas_transformation", b"koalas_transformation", "pandas_on_spark_transformation", b"pandas_on_spark_transformation", "pyspark_transformation", b"pyspark_transformation", "sql_transformation", b"sql_transformation", "type", b"type", "udf_transformation", b"udf_transformation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["udf_transformation", "sql_transformation", "koalas_transformation", "pyspark_transformation", "pandas_on_spark_transformation"] | None: ...

global___Transformation = Transformation

class UdfTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_NAME_FIELD_NUMBER: builtins.int
    function_name: builtins.str
    """The user defined function name"""
    def __init__(
        self,
        *,
        function_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["function_name", b"function_name"]) -> None: ...

global___UdfTransformation = UdfTransformation

class SqlTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SQL_FIELD_NUMBER: builtins.int
    FUNCTION_NAMES_FIELD_NUMBER: builtins.int
    sql: builtins.str
    """Sql string to run on the input data frame"""
    @property
    def function_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The user defined functions names"""
    def __init__(
        self,
        *,
        sql: builtins.str = ...,
        function_names: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["function_names", b"function_names", "sql", b"sql"]) -> None: ...

global___SqlTransformation = SqlTransformation

class KoalasTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_NAME_FIELD_NUMBER: builtins.int
    QWARGS_FIELD_NUMBER: builtins.int
    function_name: builtins.str
    """User defined Koalas function name"""
    @property
    def qwargs(self) -> global___TransformArguments: ...
    def __init__(
        self,
        *,
        function_name: builtins.str = ...,
        qwargs: global___TransformArguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["args_option", b"args_option", "qwargs", b"qwargs"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["args_option", b"args_option", "function_name", b"function_name", "qwargs", b"qwargs"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["args_option", b"args_option"]) -> typing_extensions.Literal["qwargs"] | None: ...

global___KoalasTransformation = KoalasTransformation

class PySparkTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_NAME_FIELD_NUMBER: builtins.int
    QWARGS_FIELD_NUMBER: builtins.int
    function_name: builtins.str
    """User defined pyspark function name"""
    @property
    def qwargs(self) -> global___TransformArguments: ...
    def __init__(
        self,
        *,
        function_name: builtins.str = ...,
        qwargs: global___TransformArguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["args_option", b"args_option", "qwargs", b"qwargs"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["args_option", b"args_option", "function_name", b"function_name", "qwargs", b"qwargs"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["args_option", b"args_option"]) -> typing_extensions.Literal["qwargs"] | None: ...

global___PySparkTransformation = PySparkTransformation

class PandasOnSparkTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_NAME_FIELD_NUMBER: builtins.int
    QWARGS_FIELD_NUMBER: builtins.int
    function_name: builtins.str
    """Used defined Pandas function name"""
    @property
    def qwargs(self) -> global___TransformArguments: ...
    def __init__(
        self,
        *,
        function_name: builtins.str = ...,
        qwargs: global___TransformArguments | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["args_option", b"args_option", "qwargs", b"qwargs"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["args_option", b"args_option", "function_name", b"function_name", "qwargs", b"qwargs"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["args_option", b"args_option"]) -> typing_extensions.Literal["qwargs"] | None: ...

global___PandasOnSparkTransformation = PandasOnSparkTransformation

class TransformArguments(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class QwargsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    QWARGS_FIELD_NUMBER: builtins.int
    @property
    def qwargs(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        qwargs: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["qwargs", b"qwargs"]) -> None: ...

global___TransformArguments = TransformArguments

class OnTheFlyFeatureSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_FIELD_NUMBER: builtins.int
    REQUIREMENTS_FIELD_NUMBER: builtins.int
    @property
    def function(self) -> global___UdfFunction:
        """Actual function code"""
    requirements: builtins.bytes
    """The needed requirements"""
    def __init__(
        self,
        *,
        function: global___UdfFunction | None = ...,
        requirements: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["function", b"function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["function", b"function", "requirements", b"requirements"]) -> None: ...

global___OnTheFlyFeatureSet = OnTheFlyFeatureSet

class Function(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SQL_FUNCTION_FIELD_NUMBER: builtins.int
    UDF_FUNCTION_FIELD_NUMBER: builtins.int
    @property
    def sql_function(self) -> global___SqlFunction: ...
    @property
    def udf_function(self) -> global___UdfFunction: ...
    def __init__(
        self,
        *,
        sql_function: global___SqlFunction | None = ...,
        udf_function: global___UdfFunction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["sql_function", b"sql_function", "type", b"type", "udf_function", b"udf_function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["sql_function", b"sql_function", "type", b"type", "udf_function", b"udf_function"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["sql_function", "udf_function"] | None: ...

global___Function = Function

class Backfill(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _FillUpMethod:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _FillUpMethodEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Backfill._FillUpMethod.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        AS_SCHEDULED: Backfill._FillUpMethod.ValueType  # 0
        SNAPSHOT: Backfill._FillUpMethod.ValueType  # 1

    class FillUpMethod(_FillUpMethod, metaclass=_FillUpMethodEnumTypeWrapper):
        """FillUp method for the backfill"""

    AS_SCHEDULED: Backfill.FillUpMethod.ValueType  # 0
    SNAPSHOT: Backfill.FillUpMethod.ValueType  # 1

    START_DATE_FIELD_NUMBER: builtins.int
    FILLUP_METHOD_FIELD_NUMBER: builtins.int
    @property
    def start_date(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Backfill start date"""
    fillup_method: global___Backfill.FillUpMethod.ValueType
    def __init__(
        self,
        *,
        start_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        fillup_method: global___Backfill.FillUpMethod.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["start_date", b"start_date"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fillup_method", b"fillup_method", "start_date", b"start_date"]) -> None: ...

global___Backfill = Backfill

class SqlFunction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SQL_FIELD_NUMBER: builtins.int
    sql: builtins.str
    def __init__(
        self,
        *,
        sql: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["sql", b"sql"]) -> None: ...

global___SqlFunction = SqlFunction

class UdfFunction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    code: builtins.bytes
    def __init__(
        self,
        *,
        code: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code", b"code"]) -> None: ...

global___UdfFunction = UdfFunction

class NewOnly(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___NewOnly = NewOnly

class TimeFrame(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MINUTES_FIELD_NUMBER: builtins.int
    VANILLA_FIELD_NUMBER: builtins.int
    AGGREGATION_FIELD_NUMBER: builtins.int
    minutes: builtins.int
    """Time frame size in minutes to read from a datasource"""
    @property
    def vanilla(self) -> global___Vanilla: ...
    @property
    def aggregation(self) -> global___Aggregation: ...
    def __init__(
        self,
        *,
        minutes: builtins.int = ...,
        vanilla: global___Vanilla | None = ...,
        aggregation: global___Aggregation | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aggregation", b"aggregation", "flavor", b"flavor", "vanilla", b"vanilla"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregation", b"aggregation", "flavor", b"flavor", "minutes", b"minutes", "vanilla", b"vanilla"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["flavor", b"flavor"]) -> typing_extensions.Literal["vanilla", "aggregation"] | None: ...

global___TimeFrame = TimeFrame

class Aggregation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AGGREGATION_POPULATION_FIELD_NUMBER: builtins.int
    @property
    def aggregation_population(self) -> global___AggregationPopulation: ...
    def __init__(
        self,
        *,
        aggregation_population: global___AggregationPopulation | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aggregation_population", b"aggregation_population", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregation_population", b"aggregation_population", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["aggregation_population"] | None: ...

global___Aggregation = Aggregation

class AggregationPopulation(google.protobuf.message.Message):
    """Apply time frame aggregation"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___AggregationPopulation = AggregationPopulation

class PopulationTimeframeNewOnly(google.protobuf.message.Message):
    """Get life time data for new only population"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___PopulationTimeframeNewOnly = PopulationTimeframeNewOnly

class PopulationTimeframe(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NEW_ONLY_FIELD_NUMBER: builtins.int
    @property
    def new_only(self) -> global___PopulationTimeframeNewOnly: ...
    def __init__(
        self,
        *,
        new_only: global___PopulationTimeframeNewOnly | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["new_only", b"new_only", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["new_only", b"new_only", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["new_only"] | None: ...

global___PopulationTimeframe = PopulationTimeframe

class Vanilla(google.protobuf.message.Message):
    """FullRead policy default value"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Vanilla = Vanilla

class FullRead(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEFAULT_FIELD_NUMBER: builtins.int
    POPULATION_TIMEFRAME_FIELD_NUMBER: builtins.int
    @property
    def default(self) -> global___Vanilla: ...
    @property
    def population_timeframe(self) -> global___PopulationTimeframe: ...
    def __init__(
        self,
        *,
        default: global___Vanilla | None = ...,
        population_timeframe: global___PopulationTimeframe | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["default", b"default", "flavor", b"flavor", "population_timeframe", b"population_timeframe"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["default", b"default", "flavor", b"flavor", "population_timeframe", b"population_timeframe"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["flavor", b"flavor"]) -> typing_extensions.Literal["default", "population_timeframe"] | None: ...

global___FullRead = FullRead

class DataSourceReadPolicy(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NEW_ONLY_FIELD_NUMBER: builtins.int
    TIME_FRAME_FIELD_NUMBER: builtins.int
    FULL_READ_FIELD_NUMBER: builtins.int
    @property
    def new_only(self) -> global___NewOnly:
        """Read only new records from the data source - each batch reads from where the last batch finished with no end limit"""
    @property
    def time_frame(self) -> global___TimeFrame:
        """Read records starting from (scheduled batch time - time defined in the time frame) ending at the scheduled batch time"""
    @property
    def full_read(self) -> global___FullRead:
        """Read all records from the data source each batch"""
    def __init__(
        self,
        *,
        new_only: global___NewOnly | None = ...,
        time_frame: global___TimeFrame | None = ...,
        full_read: global___FullRead | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["full_read", b"full_read", "new_only", b"new_only", "time_frame", b"time_frame", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["full_read", b"full_read", "new_only", b"new_only", "time_frame", b"time_frame", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["new_only", "time_frame", "full_read"] | None: ...

global___DataSourceReadPolicy = DataSourceReadPolicy
